{% extends "base.html" %}

{% block title %}{{ post.title }} - {{ site.name }}{% endblock %}

{% block meta %}
{{ seo_meta | safe }}
{% endblock %}

{% block content %}
<article class="max-w-3xl mx-auto">
    <header class="mb-8">
        <h1 class="text-3xl font-bold text-gray-900 mb-2">{{ post.title }}</h1>
        <div class="text-sm text-gray-500">
            <time datetime="{{ post.created_at }}">{{ post.created_at[:10] }}</time>
        </div>
    </header>

    <div class="prose prose-lg max-w-none">
        {{ post.body | render_blocks | safe }}
    </div>
</article>
{% endblock %}
