<!DOCTYPE html>
<html lang="{{ site.language | default('ja') }}">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{% block title %}{{ site.name }}{% endblock %}</title>
    <script src="https://cdn.tailwindcss.com"></script>
</head>
<body class="bg-gray-50 min-h-screen">
    <header class="bg-white shadow">
        <div class="max-w-4xl mx-auto px-4 py-6">
            <h1 class="text-2xl font-bold text-gray-900">
                <a href="/">{{ site.name }}</a>
            </h1>
            {% if site.tagline %}
            <p class="text-gray-600">{{ site.tagline }}</p>
            {% endif %}
        </div>
    </header>

    <main class="max-w-4xl mx-auto px-4 py-8">
        {% block content %}{% endblock %}
    </main>

    <footer class="bg-gray-100 mt-auto">
        <div class="max-w-4xl mx-auto px-4 py-6 text-center text-gray-600">
            <p>&copy; {{ now().year }} {{ site.name }}. Powered by Focomy.</p>
        </div>
    </footer>
</body>
</html>
