{% extends "base.html" %}

{% block title %}{{ page.title }} - {{ site_name }}{% endblock %}

{% block meta %}
{{ seo_meta | safe }}
{% endblock %}

{% block content %}
<article class="page">
    <header class="post-header">
        <h1>{{ page.title }}</h1>
    </header>

    <div class="post-content">
        {{ page.body | render_blocks | safe }}
    </div>
</article>
{% endblock %}
